import { PIXI } from "../pixi";
import { getRandomArrayItem } from "../utils/random";

export type CharacterSpriteAnimationFrameMetadata = {
  frame: 0 | 1 | 2 | 3;
  isAnimated: boolean;
  nextFrameIn: number;
  animationSpeed: number;
};

type SharedCharacterSpriteData = {
  direction: "n" | "s" | "e" | "w";
  state?: "struggle";
  status?: "shouting" | "warning";
  struggleDataPercent?: number;
  textData?:
    | "gearUp"
    | "untie"
    | "rob"
    | "struggle"
    | "capture"
    | "deliver"
    | "pickUpDelivery";
  clothing: "dress" | "armor" | "underwear";
  displayedMoney?: number;
  selected?: boolean;
  eyesColor: number;
  clothesColor: number;
  underwearColor: number;
  hairColor: number;
};

export type CharacterSpriteRenderUpdaterComponent =
  SharedCharacterSpriteData & {
    container: PIXI.Container;
    movement: CharacterSpriteAnimationFrameMetadata;
    expression: CharacterSpriteAnimationFrameMetadata;
    drawnSprite?: SharedCharacterSpriteData & {
      movement: 0 | 1 | 2 | 3;
      expression: 0 | 1 | 2 | 3;
    };
  };

export function createCharacterSpriteRendererUpdaterComponent(
  container: PIXI.Container,
): CharacterSpriteRenderUpdaterComponent {
  const clothesColor = getRandomArrayItem([
    0xff5533, 0x33aa11, 0x5599dd, 0xcc88dd,
  ]);
  const underwearColor = getRandomArrayItem([
    0xff5533, 0x33aa11, 0x5599dd, 0xcc88dd,
  ]);
  const eyesColor = getRandomArrayItem([
    0x5577ff, 0x99ccff, 0x33aa33, 0xaa5533, 0xdd7755,
  ]);
  const hairColor = getRandomArrayItem([
    0xeeee55, 0x553333, 0xeeaa33, 0xee7733,
  ]);
  return {
    container,
    direction: "s",
    movement: {
      frame: 0,
      isAnimated: false,
      nextFrameIn: 20,
      animationSpeed: 4,
    },
    expression: {
      frame: 0,
      isAnimated: true,
      nextFrameIn: 20,
      animationSpeed: 4,
    },
    clothing: "dress",
    eyesColor,
    clothesColor,
    underwearColor,
    hairColor,
  };
}
