export type TimerMetadata = {
  time: number;
  current: number;
};

export type ControlsPlayerComponent = {
  isBandit: boolean;
  walkSpeed: number;
  sprintSpeed: number;
  equip: TimerMetadata;
  equipRotationSpeed: number;
  captureAttempt: TimerMetadata;
  capture: TimerMetadata;
  captureTarget?: string;
  captureDistanceFront: number;
  captureDistanceSides: number;
  captureStrength: number;
  freeTarget?: string;
  freeDistanceFront: number;
  freeDistanceSides: number;
  robTarget?: string;
  robDistanceFront: number;
  robDistanceSides: number;
  npcInteraction: TimerMetadata;
  npcInteractionTarget?: string;
  npcInteractionDistanceFront: number;
  npcInteractionDistanceSides: number;
  preventMovingOffRoad: boolean;
  preventWalkingOverRestrainedAllies: boolean;
  preventCapturingOthers: boolean;
  pressedAction1KeyThisFrame: boolean;
  holdingAction1Key: boolean;
  undressingTime: number;
  remainingUndressingTime: number;
  lastStruggleKey: {
    up: boolean;
    down: boolean;
    left: boolean;
    right: boolean;
  };
};

export function createControlsPlayerComponent(params: {
  isBandit: boolean;
}): ControlsPlayerComponent {
  return {
    isBandit: params.isBandit,
    walkSpeed: 0.2,
    sprintSpeed: 0.3,
    equip: { time: 40, current: 0 },
    equipRotationSpeed: 1 / 8,
    captureAttempt: { time: 40, current: 0 },
    capture: { time: 40, current: 0 },
    captureDistanceFront: 2.5,
    captureDistanceSides: 1.5,
    captureStrength: 100,
    freeDistanceFront: 1.5,
    freeDistanceSides: 0.5,
    robDistanceFront: 1.5,
    robDistanceSides: 0.5,
    npcInteraction: { time: 20, current: 0 },
    npcInteractionDistanceFront: 1.5,
    npcInteractionDistanceSides: 1,
    undressingTime: 30,
    remainingUndressingTime: 0,
    pressedAction1KeyThisFrame: false,
    holdingAction1Key: false,
    preventMovingOffRoad: true,
    preventWalkingOverRestrainedAllies: true,
    preventCapturingOthers: false,
    lastStruggleKey: {
      up: false,
      down: false,
      left: false,
      right: false,
    },
  };
}
