import { ImageRenderingData } from "../images";
import { PIXI } from "../pixi";

export type UIPlayerUpdaterComponent = {
  showBanditUI: boolean;
  totalAcquiredMoney: number;
  totalTownieMoney: number;
  totalStolenMoney: number;
  currentTowniesToSave: number;
  currentBanditsToSave: number;
  canCapture: boolean;
  captureAttemptPercent: number;
  delivery:
    | { state: "noDeliveryNoGiver" }
    | { state: "noDelivery"; direction: number }
    | {
        state: "delivery";
        packages: Array<{ name: string; value: number; direction: number }>;
      };
  /** Used for rendering */
  pixi: {
    acquiredMoney: PIXI.Text;
    allyMoney: PIXI.Text;
    stolenMoney: PIXI.Text;
    alliesToSave: PIXI.Text;
    deliveryContainer: PIXI.Container;
    deliveryPackageText: PIXI.Text;
    deliveryArrows: PIXI.Sprite[];
    noDeliveryContainer: PIXI.Container;
    noDeliveryArrow: PIXI.Sprite;
    captureContainer: PIXI.Container;
    captureContainerPercentBar: PIXI.Graphics;
    noCaptureContainer: PIXI.Container;
  };
};

export function createUIPlayerUpdaterComponent(
  showBanditUI: boolean,
  container: PIXI.Container,
  images: ImageRenderingData,
): UIPlayerUpdaterComponent {
  const pixi: UIPlayerUpdaterComponent["pixi"] = {
    acquiredMoney: new PIXI.Text({
      text: "",
      style: { fill: "white", fontSize: 18 },
    }),
    allyMoney: new PIXI.Text({
      text: "",
      style: { fill: "white", fontSize: 18 },
    }),
    stolenMoney: new PIXI.Text({
      text: "",
      style: { fill: "white", fontSize: 18 },
    }),
    alliesToSave: new PIXI.Text({
      text: "",
      style: { fill: "white", fontSize: 18 },
    }),
    deliveryContainer: new PIXI.Container(),
    deliveryPackageText: new PIXI.Text({ text: "", style: { fontSize: 18 } }),
    deliveryArrows: [],
    noDeliveryContainer: new PIXI.Container(),
    noDeliveryArrow: new PIXI.Sprite(images.cursor),
    captureContainer: new PIXI.Container(),
    captureContainerPercentBar: new PIXI.Graphics(),
    noCaptureContainer: new PIXI.Container(),
  };
  container.addChild(pixi.acquiredMoney);
  pixi.allyMoney.y = 30;
  container.addChild(pixi.allyMoney);
  pixi.stolenMoney.y = 60;
  container.addChild(pixi.stolenMoney);
  pixi.alliesToSave.y = 90;
  container.addChild(pixi.alliesToSave);

  const captureContainerText = new PIXI.Text({
    text: `Capture: `,
    style: {
      fontSize: 24,
    },
  });
  captureContainerText.y = 125;
  pixi.captureContainer.addChild(captureContainerText);
  const captureContainerPercentBarBorder = new PIXI.Graphics();
  captureContainerPercentBarBorder.rect(2, 2, 196, 16);
  captureContainerPercentBarBorder.fill(0xffffff);
  captureContainerPercentBarBorder.stroke({ width: 2, color: 0xcccccc });
  captureContainerPercentBarBorder.x = 100;
  captureContainerPercentBarBorder.y = 130;
  pixi.captureContainer.addChild(captureContainerPercentBarBorder);
  pixi.captureContainerPercentBar.x = 100;
  pixi.captureContainerPercentBar.y = 130;
  pixi.captureContainer.addChild(pixi.captureContainerPercentBar);
  container.addChild(pixi.captureContainer);
  const noCaptureContainerText = new PIXI.Text({
    text: `Capture unavailable (no equipment)`,
    style: { fontSize: 24 },
  });
  noCaptureContainerText.y = 125;
  pixi.noCaptureContainer.addChild(noCaptureContainerText);
  container.addChild(pixi.noCaptureContainer);

  const deliveryContainerText = new PIXI.Text({
    text: `Deliveries: `,
    style: { fontSize: 24 },
  });
  deliveryContainerText.y = 185;
  pixi.deliveryContainer.addChild(deliveryContainerText);
  let offset = 130;
  for (let i = 0; i < 5; ++i) {
    const arrowSprite = new PIXI.Sprite(images.cursor);
    arrowSprite.anchor.set(0.5, 0.5);
    arrowSprite.rotation = 0;
    arrowSprite.x = offset;
    arrowSprite.y = 200;
    pixi.deliveryContainer.addChild(arrowSprite);
    pixi.deliveryArrows.push(arrowSprite);
    offset += 32;
  }
  pixi.deliveryPackageText.y = 220;
  pixi.deliveryContainer.addChild(pixi.deliveryPackageText);
  container.addChild(pixi.deliveryContainer);

  const noDeliveryContainerText = new PIXI.Text({
    text: `Nearest Delivery: `,
    style: { fontSize: 24 },
  });
  noDeliveryContainerText.y = 185;
  pixi.noDeliveryContainer.addChild(noDeliveryContainerText);
  pixi.noDeliveryArrow.anchor.set(0.5, 0.5);
  pixi.noDeliveryArrow.x = 200;
  pixi.noDeliveryArrow.y = 200;
  pixi.noDeliveryContainer.addChild(pixi.noDeliveryArrow);
  container.addChild(pixi.noDeliveryContainer);

  return {
    showBanditUI,
    totalAcquiredMoney: 0,
    totalTownieMoney: 0,
    totalStolenMoney: 0,
    currentTowniesToSave: 0,
    currentBanditsToSave: 0,
    canCapture: false,
    captureAttemptPercent: 1,
    delivery: { state: "noDelivery", direction: 0 },
    pixi,
  };
}
