import { EntityType } from ".";
import { createAIAllyComponent } from "../components/ai-ally";
import { createCapturableComponent } from "../components/capturable";
import { createCharacterMapMovementComponent } from "../components/map-movement";
import { createMapPathingComponent } from "../components/map-pathing";
import { createCharacterSpriteRendererUpdaterComponent } from "../components/character-sprite-render-updater";
import { createInventoryComponent } from "../components/inventory";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

export function createAllyEntity(
  targetTown: string,
  path: Array<{ x: number; y: number }>,
  tileSize: number,
): EntityType {
  const container = new PIXI.Container();
  const x = path[0].x;
  const y = path[0].y;
  return {
    id: generateUUID(),
    type: "ally",
    aiAlly: createAIAllyComponent(targetTown),
    capturable: createCapturableComponent({
      banditEnemy: true,
      townieRescuable: true,
    }),
    mapPathing: createMapPathingComponent(path),
    mapMovement: createCharacterMapMovementComponent(x, y, 0.05),
    characterSpriteRenderUpdater:
      createCharacterSpriteRendererUpdaterComponent(container),
    inventory: createInventoryComponent("dress", 10),
    mapPosition: createMapPositionComponent(x, y),
    mapRenderer: createRendererComponentWithAnchor(container, {
      x: -tileSize / 2,
      y: -tileSize - tileSize / 2,
    }),
  };
}
