import { EntityType } from ".";
import { createSpawnerEnemyComponent } from "../components/spawner-enemy";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { Game } from "../engine";
import { CampData } from "../map";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

function getTileFromCamp(camp: CampData): string {
  if (camp.image === "fire") return "fire";
  return "camp";
}

export function createCampEntity(
  game: Game,
  camp: CampData,
  tileSize: number,
): EntityType {
  const sprite = new PIXI.Sprite(
    game.images.tilesLarge.textures[getTileFromCamp(camp)],
  );

  return {
    id: generateUUID(),
    type: "camp",
    spawnerEnemy: createSpawnerEnemyComponent(),
    mapPosition: createMapPositionComponent(camp.x + 1, camp.y + 1),
    mapRenderer: createRendererComponentWithAnchor(sprite, {
      x: -tileSize,
      y: -tileSize,
    }),
  };
}
