import { EntityType } from ".";
import { createAIDeliveryComponent } from "../components/ai-delivery";
import { createCharacterSpriteRendererUpdaterComponent } from "../components/character-sprite-render-updater";
import { createInteractibleNPCComponent } from "../components/interactible-npc";
import { createInventoryComponent } from "../components/inventory";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

export function createDeliveryTargetEntity(
  x: number,
  y: number,
  townId: string,
  tileSize: number,
): EntityType {
  const container = new PIXI.Container();
  const inventory = createInventoryComponent("dress", 0);
  return {
    id: generateUUID(),
    type: "deliveryTarget",
    aiDelivery: createAIDeliveryComponent({ rotation: 40 }),
    characterSpriteRenderUpdater:
      createCharacterSpriteRendererUpdaterComponent(container),
    inventory: inventory,
    interactibleNPC: createInteractibleNPCComponent("deliver", townId),
    mapPosition: createMapPositionComponent(x, y),
    mapRenderer: createRendererComponentWithAnchor(container, {
      x: -tileSize / 2,
      y: -tileSize - tileSize / 2,
    }),
  };
}
