import { EntityType } from ".";
import { createAIEnemyComponent } from "../components/ai-enemy";
import { createCapturableComponent } from "../components/capturable";
import { createCharacterMapMovementComponent } from "../components/map-movement";
import { createMapPathingComponent } from "../components/map-pathing";
import { createCharacterSpriteRendererUpdaterComponent } from "../components/character-sprite-render-updater";
import { createInventoryComponent } from "../components/inventory";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

export function createEnemyEntity(
  targetCamp: string,
  path: Array<{ x: number; y: number }>,
  tileSize: number,
): EntityType {
  const container = new PIXI.Container();
  const x = path[0].x;
  const y = path[0].y;
  const characterSpriteRenderUpdater =
    createCharacterSpriteRendererUpdaterComponent(container);
  characterSpriteRenderUpdater.clothing = "armor";
  characterSpriteRenderUpdater.clothesColor = 0x666666;
  return {
    id: generateUUID(),
    type: "enemy",
    aiEnemy: createAIEnemyComponent(targetCamp),
    capturable: createCapturableComponent({
      townieEnemy: true,
      banditRescuable: true,
    }),
    mapPathing: createMapPathingComponent(path),
    mapMovement: createCharacterMapMovementComponent(x, y, 0.1),
    characterSpriteRenderUpdater,
    inventory: createInventoryComponent("armor", 0),
    mapPosition: createMapPositionComponent(x, y),
    mapRenderer: createRendererComponentWithAnchor(container, {
      x: -tileSize / 2,
      y: -tileSize - tileSize / 2,
    }),
  };
}
