import { EntityType } from ".";
import { createCapturableComponent } from "../components/capturable";
import { createCharacterMapMovementComponent } from "../components/map-movement";
import { createMapPathingComponent } from "../components/map-pathing";
import { createCharacterSpriteRendererUpdaterComponent } from "../components/character-sprite-render-updater";
import { createInventoryComponent } from "../components/inventory";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";
import { createAIGuardComponent } from "../components/ai-guard";

export function createGuardEntity(
  targetTown: string,
  path: Array<{ x: number; y: number }>,
  tileSize: number,
): EntityType {
  const container = new PIXI.Container();
  const x = path[0].x;
  const y = path[0].y;
  const characterSpriteRenderUpdater =
    createCharacterSpriteRendererUpdaterComponent(container);
  characterSpriteRenderUpdater.clothing = "armor";
  characterSpriteRenderUpdater.clothesColor = 0xaaaaaa;
  return {
    id: generateUUID(),
    type: "ally",
    aiGuard: createAIGuardComponent(targetTown),
    capturable: createCapturableComponent({
      banditEnemy: true,
      townieRescuable: true,
    }),
    mapPathing: createMapPathingComponent(path),
    mapMovement: createCharacterMapMovementComponent(x, y, 0.08),
    characterSpriteRenderUpdater,
    inventory: createInventoryComponent("armor", 5),
    mapPosition: createMapPositionComponent(x, y),
    mapRenderer: createRendererComponentWithAnchor(container, {
      x: -tileSize / 2,
      y: -tileSize - tileSize / 2,
    }),
  };
}
