import { EntityType } from ".";
import { createMapMetadataComponent } from "../components/map-metadata";
import { createRendererComponent } from "../components/renderer";
import { MapData, RoadData } from "../map";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

function getTileFromRoad(road: RoadData): string {
  let result = "road";
  if (road.north) result += "N";
  if (road.south) result += "S";
  if (road.east) result += "E";
  if (road.west) result += "W";
  return result;
}

export function createMapEntity(
  map: MapData,
  graph: Array<[string, string]>,
  spritesheet: PIXI.Spritesheet,
): EntityType {
  const container = new PIXI.Container();
  const backgroundContainer = new PIXI.Container();
  for (let x = 0; x < map.sizeX; ++x) {
    for (let y = 0; y < map.sizeY; ++y) {
      const sprite = new PIXI.Sprite(spritesheet.textures.grass);
      sprite.x = map.tileSize * x;
      sprite.y = map.tileSize * y;
      backgroundContainer.addChild(sprite);
    }
  }
  container.addChild(backgroundContainer);
  for (const road of map.roads) {
    const sprite = new PIXI.Sprite(spritesheet.textures[getTileFromRoad(road)]);
    sprite.x = map.tileSize * road.x;
    sprite.y = map.tileSize * road.y;
    container.addChild(sprite);
  }

  return {
    id: generateUUID(),
    type: "other",
    mapMetadata: createMapMetadataComponent(map, graph),
    mapRenderer: createRendererComponent(container),
  };
}
