import { EntityType } from ".";
import { createCharacterSpriteRendererUpdaterComponent } from "../components/character-sprite-render-updater";
import { createCharacterMapMovementComponent } from "../components/map-movement";
import { createMapPositionComponent } from "../components/map-position";
import {
  createRendererComponent,
  createRendererComponentWithAnchor,
} from "../components/renderer";
import { PIXI } from "../pixi";
import { createControlsPlayerComponent } from "../components/controls-player";
import { createMapCameraPositionComponent } from "../components/camera-position";
import { createCapturableComponent } from "../components/capturable";
import { generateUUID } from "../utils/uuid";
import { createInventoryComponent } from "../components/inventory";
import { createUIPlayerUpdaterComponent } from "../components/ui-player-updater";
import { ImageRenderingData } from "../images";

export function createPlayerEntity(
  x: number,
  y: number,
  images: ImageRenderingData,
  tileSize: number,
): EntityType {
  const container = new PIXI.Container();
  const characterSpriteRenderUpdater =
    createCharacterSpriteRendererUpdaterComponent(container);
  characterSpriteRenderUpdater.clothing = "dress";
  characterSpriteRenderUpdater.clothesColor = 0x9999bb;
  characterSpriteRenderUpdater.selected = true;
  const uiContainer = new PIXI.Container();
  return {
    id: generateUUID(),
    type: "player",
    capturable: createCapturableComponent({ banditEnemy: true }),
    mapCameraPosition: createMapCameraPositionComponent(),
    mapMovement: createCharacterMapMovementComponent(x, y, 0.2),
    controlsPlayer: createControlsPlayerComponent({ isBandit: false }),
    characterSpriteRenderUpdater,
    inventory: createInventoryComponent("dress", 10),
    mapPosition: createMapPositionComponent(x, y),
    mapRenderer: createRendererComponentWithAnchor(container, {
      x: -tileSize / 2,
      y: -tileSize - tileSize / 2,
    }),
    uiPlayerUpdater: createUIPlayerUpdaterComponent(false, uiContainer, images),
    uiRenderer: createRendererComponent(uiContainer),
  };
}
