import { EntityType } from ".";
import { createControlsTitleScreenComponent } from "../components/controls-title-screen";
import { createRendererComponent } from "../components/renderer";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

export function createTitleScreen(params: { banditMode: boolean }): EntityType {
  const container = new PIXI.Container();
  const title = new PIXI.Text({
    text: params.banditMode
      ? "Strand-type DiD Game (Bandit mode!)"
      : "Strand-type DiD Game",
    style: {
      fill: "white",
      fontSize: 36,
    },
  });
  title.x = 400;
  title.y = 50;
  title.anchor.set(0.5, 0.5);
  container.addChild(title);
  let instructionsText = "";
  if (params.banditMode) {
    instructionsText = `(On mobile, tap to show controls)
    
  Goal:
  Capture delivery girls
  Rob the girls you catch
  Help out your fellow bandits
  Try not to get caught by the guards
  
  Controls:
  [Arrows] or [WASD] to move
  [Shift] to sprint
  Use [SPACE] or [ENTER] for actions
  (Pick up or deliver package, free a tied-up girl)
  Tie up an enemy with [Action] (they'll eventually struggle out)
  (Hint: guards can't see you when you sneak behind them)
  [Arrows] or [WASD] to struggle`;
  } else {
    instructionsText = `(On mobile, tap to show controls)
      
  Goal:
  Deliver packages
  Protect as many travelling girls as you can
  Rescue the girls that got captured
  Try not to get captured yourself
  
  Controls:
  [Arrows] or [WASD] to move
  [Shift] to sprint
  Use [SPACE] or [ENTER] for actions
  (Pick up or deliver package, free a tied-up girl)
  Tie up an enemy with [Action] (they'll eventually struggle out)
  (Hint: Enemies can't see you when you sneak behind them)
  (Note: You can't tie up enemies without equipment!)
  [Arrows] or [WASD] to struggle`;
  }
  const instructions = new PIXI.Text({
    text: instructionsText,
    style: { align: "center", fill: "white", fontSize: 20 },
  });
  instructions.x = 400;
  instructions.y = 280;
  instructions.anchor.set(0.5, 0.5);
  container.addChild(instructions);
  const start = new PIXI.Text({
    text: `Press [SPACE] to start`,
    style: { align: "center", fill: "white", fontSize: 36 },
  });
  start.x = 400;
  start.y = 500;
  start.anchor.set(0.5, 0);
  container.addChild(start);
  return {
    id: generateUUID(),
    type: "other",
    controlsTitleScreen: createControlsTitleScreenComponent(params),
    uiRenderer: createRendererComponent(container),
  };
}
