import { EntityType } from ".";
import { createSpawnerAllyComponent } from "../components/spawner-ally";
import { createCapturePreventionComponent } from "../components/capture-prevention";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { Game } from "../engine";
import { TownData } from "../map";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";
import { createSpawnerDeliveryComponent } from "../components/spawner-delivery";

function getTileFromTown(town: TownData): string {
  if (town.image === "red") return "townRed";
  return "townBlue";
}

export function createHouseEntity(
  game: Game,
  town: TownData,
  tileSize: number,
): EntityType {
  const sprite = new PIXI.Sprite(
    game.images.tilesLarge.textures[getTileFromTown(town)],
  );

  return {
    id: generateUUID(),
    type: "house",
    spawnerAlly: createSpawnerAllyComponent(),
    spawnerDelivery: createSpawnerDeliveryComponent(),
    capturePrevention: createCapturePreventionComponent(2),
    mapPosition: createMapPositionComponent(town.x + 1, town.y + 0.5),
    mapRenderer: createRendererComponentWithAnchor(sprite, {
      x: -tileSize,
      y: -tileSize,
    }),
  };
}
