import { EntityType } from ".";
import { createMapPositionComponent } from "../components/map-position";
import { createRendererComponentWithAnchor } from "../components/renderer";
import { Game } from "../engine";
import { TreeData } from "../map";
import { PIXI } from "../pixi";
import { generateUUID } from "../utils/uuid";

function getTileFromTree(tree: TreeData): string {
  if (tree.tile === "1") return "tree1";
  return "tree2";
}

export function createTreeEntity(
  game: Game,
  tree: TreeData,
  tileSize: number,
): EntityType {
  const sprite = new PIXI.Sprite(
    game.images.tilesLarge.textures[getTileFromTree(tree)],
  );

  return {
    id: generateUUID(),
    type: "other",
    mapPosition: createMapPositionComponent(tree.x + 0.5, tree.y + 0.5),
    mapRenderer: createRendererComponentWithAnchor(sprite, {
      x: -tileSize / 2,
      y: -tileSize,
    }),
  };
}
