import { PIXI } from "../pixi";

export type CharacterRenderingData = {
  body: PIXI.Spritesheet;
  eyes: PIXI.Spritesheet;
  hair: PIXI.Spritesheet;
  underwear: PIXI.Spritesheet;
  dress: PIXI.Spritesheet;
  armor: PIXI.Spritesheet;
  binds: PIXI.Spritesheet;
  expressions: PIXI.Spritesheet;
};

export type ImageRenderingData = {
  cursor: PIXI.Texture;
  goldCoin: PIXI.Texture;
  tilesSmall: PIXI.Spritesheet;
  tilesLarge: PIXI.Spritesheet;
  characterSmall: CharacterRenderingData;
  characterLarge: CharacterRenderingData;
};

export async function loadAllSpritesheets(): Promise<ImageRenderingData> {
  const tilesAtlas: PIXI.SpritesheetData = {
    frames: {
      grass: { frame: { x: 0, y: 80, w: 20, h: 20 } },
      roadNSEW: { frame: { x: 100, y: 0, w: 20, h: 20 } },
      roadNSE: { frame: { x: 0, y: 20, w: 20, h: 20 } },
      roadNSW: { frame: { x: 20, y: 20, w: 20, h: 20 } },
      roadNEW: { frame: { x: 80, y: 20, w: 20, h: 20 } },
      roadNS: { frame: { x: 0, y: 0, w: 20, h: 20 } },
      roadNE: { frame: { x: 40, y: 20, w: 20, h: 20 } },
      roadNW: { frame: { x: 60, y: 20, w: 20, h: 20 } },
      roadSEW: { frame: { x: 80, y: 0, w: 20, h: 20 } },
      roadSE: { frame: { x: 40, y: 0, w: 20, h: 20 } },
      roadSW: { frame: { x: 60, y: 0, w: 20, h: 20 } },
      roadEW: { frame: { x: 20, y: 0, w: 20, h: 20 } },
      townRed: { frame: { x: 80, y: 40, w: 40, h: 40 } },
      townBlue: { frame: { x: 40, y: 40, w: 40, h: 40 } },
      camp: { frame: { x: 40, y: 80, w: 40, h: 40 } },
      fire: { frame: { x: 80, y: 80, w: 40, h: 40 } },
      tree1: { frame: { x: 0, y: 40, w: 20, h: 40 } },
      tree2: { frame: { x: 20, y: 40, w: 20, h: 40 } },
    },
    meta: { scale: "1" },
  };
  const tilesLargeAtlas: PIXI.SpritesheetData = {
    frames: {
      grass: { frame: { x: 0, y: 168, w: 42, h: 42 } },
      roadNSEW: { frame: { x: 210, y: 0, w: 42, h: 42 } },
      roadNSE: { frame: { x: 0, y: 42, w: 42, h: 42 } },
      roadNSW: { frame: { x: 42, y: 42, w: 42, h: 42 } },
      roadNEW: { frame: { x: 168, y: 42, w: 42, h: 42 } },
      roadNS: { frame: { x: 0, y: 0, w: 42, h: 42 } },
      roadNE: { frame: { x: 84, y: 42, w: 42, h: 42 } },
      roadNW: { frame: { x: 126, y: 42, w: 42, h: 42 } },
      roadSEW: { frame: { x: 168, y: 0, w: 42, h: 42 } },
      roadSE: { frame: { x: 84, y: 0, w: 42, h: 42 } },
      roadSW: { frame: { x: 126, y: 0, w: 42, h: 42 } },
      roadEW: { frame: { x: 42, y: 0, w: 42, h: 42 } },
      townRed: { frame: { x: 168, y: 84, w: 84, h: 84 } },
      townBlue: { frame: { x: 84, y: 84, w: 84, h: 84 } },
      camp: { frame: { x: 84, y: 168, w: 84, h: 84 } },
      fire: { frame: { x: 168, y: 168, w: 84, h: 84 } },
      tree1: { frame: { x: 0, y: 84, w: 42, h: 84 } },
      tree2: { frame: { x: 42, y: 84, w: 42, h: 84 } },
    },
    meta: { scale: "1" },
  };
  const characterAtlas: PIXI.SpritesheetData = {
    frames: {
      walkS1: { frame: { x: 0, y: 0, w: 28, h: 38 } },
      walkS2: { frame: { x: 28, y: 0, w: 28, h: 38 } },
      walkS3: { frame: { x: 56, y: 0, w: 28, h: 38 } },
      walkW1: { frame: { x: 0, y: 38, w: 28, h: 38 } },
      walkW2: { frame: { x: 28, y: 38, w: 28, h: 38 } },
      walkW3: { frame: { x: 56, y: 38, w: 28, h: 38 } },
      walkE1: { frame: { x: 0, y: 76, w: 28, h: 38 } },
      walkE2: { frame: { x: 28, y: 76, w: 28, h: 38 } },
      walkE3: { frame: { x: 56, y: 76, w: 28, h: 38 } },
      walkN1: { frame: { x: 0, y: 114, w: 28, h: 38 } },
      walkN2: { frame: { x: 28, y: 114, w: 28, h: 38 } },
      walkN3: { frame: { x: 56, y: 114, w: 28, h: 38 } },
      struggle1: { frame: { x: 0, y: 152, w: 28, h: 38 } },
      struggle2: { frame: { x: 28, y: 152, w: 28, h: 38 } },
      struggle3: { frame: { x: 56, y: 152, w: 28, h: 38 } },
    },
    animations: {
      walkS: ["walkS1", "walkS2", "walkS3", "walkS2"],
      walkW: ["walkW1", "walkW2", "walkW3", "walkW2"],
      walkE: ["walkE1", "walkE2", "walkE3", "walkE2"],
      walkN: ["walkN1", "walkN2", "walkN3", "walkN2"],
      struggle: ["struggle1", "struggle2", "struggle3", "struggle2"],
    },
    meta: { scale: "1" },
  };
  const characterExpressionAtlas: PIXI.SpritesheetData = {
    frames: {
      shout1: { frame: { x: 0, y: 0, w: 28, h: 38 } },
      shout2: { frame: { x: 28, y: 0, w: 28, h: 38 } },
      shout3: { frame: { x: 56, y: 0, w: 28, h: 38 } },
      warning1: { frame: { x: 0, y: 38, w: 28, h: 38 } },
      warning2: { frame: { x: 28, y: 38, w: 28, h: 38 } },
      warning3: { frame: { x: 56, y: 38, w: 28, h: 38 } },
      select1: { frame: { x: 0, y: 76, w: 28, h: 38 } },
      select2: { frame: { x: 28, y: 76, w: 28, h: 38 } },
      select3: { frame: { x: 56, y: 76, w: 28, h: 38 } },
    },
    animations: {
      shout: ["shout1", "shout2", "shout3", "shout2"],
      warning: ["warning1", "warning2", "warning3", "warning2"],
      select: ["select1", "select2", "select3", "select2"],
    },
    meta: { scale: "1" },
  };
  const characterLargeAtlas: PIXI.SpritesheetData = {
    frames: {
      walkS1: { frame: { x: 0, y: 0, w: 84, h: 84 } },
      walkS2: { frame: { x: 84, y: 0, w: 84, h: 84 } },
      walkS3: { frame: { x: 168, y: 0, w: 84, h: 84 } },
      walkW1: { frame: { x: 0, y: 84, w: 84, h: 84 } },
      walkW2: { frame: { x: 84, y: 84, w: 84, h: 84 } },
      walkW3: { frame: { x: 168, y: 84, w: 84, h: 84 } },
      walkE1: { frame: { x: 0, y: 168, w: 84, h: 84 } },
      walkE2: { frame: { x: 84, y: 168, w: 84, h: 84 } },
      walkE3: { frame: { x: 168, y: 168, w: 84, h: 84 } },
      walkN1: { frame: { x: 0, y: 252, w: 84, h: 84 } },
      walkN2: { frame: { x: 84, y: 252, w: 84, h: 84 } },
      walkN3: { frame: { x: 168, y: 252, w: 84, h: 84 } },
      struggle1: { frame: { x: 0, y: 336, w: 84, h: 84 } },
      struggle2: { frame: { x: 84, y: 336, w: 84, h: 84 } },
      struggle3: { frame: { x: 168, y: 336, w: 84, h: 84 } },
    },
    animations: {
      walkS: ["walkS1", "walkS2", "walkS3", "walkS2"],
      walkW: ["walkW1", "walkW2", "walkW3", "walkW2"],
      walkE: ["walkE1", "walkE2", "walkE3", "walkE2"],
      walkN: ["walkN1", "walkN2", "walkN3", "walkN2"],
      struggle: ["struggle1", "struggle2", "struggle3", "struggle2"],
    },
    meta: { scale: "1" },
  };
  const characterLargeExpressionAtlas: PIXI.SpritesheetData = {
    frames: {
      shout1: { frame: { x: 0, y: 0, w: 84, h: 84 } },
      shout2: { frame: { x: 84, y: 0, w: 84, h: 84 } },
      shout3: { frame: { x: 168, y: 0, w: 84, h: 84 } },
      warning1: { frame: { x: 0, y: 84, w: 84, h: 84 } },
      warning2: { frame: { x: 84, y: 84, w: 84, h: 84 } },
      warning3: { frame: { x: 168, y: 84, w: 84, h: 84 } },
      select1: { frame: { x: 0, y: 168, w: 84, h: 84 } },
      select2: { frame: { x: 84, y: 168, w: 84, h: 84 } },
      select3: { frame: { x: 168, y: 168, w: 84, h: 84 } },
    },
    animations: {
      shout: ["shout1", "shout2", "shout3", "shout2"],
      warning: ["warning1", "warning2", "warning3", "warning2"],
      select: ["select1", "select2", "select3", "select2"],
    },
    meta: { scale: "1" },
  };
  const data: ImageRenderingData = {
    cursor: await PIXI.Assets.load("images/cursor.png"),
    goldCoin: await PIXI.Assets.load("images/gold_coin.png"),
    tilesSmall: new PIXI.Spritesheet(
      await PIXI.Assets.load("images/tiles_small.png"),
      tilesAtlas,
    ),
    tilesLarge: new PIXI.Spritesheet(
      await PIXI.Assets.load("images/tiles_large.png"),
      tilesLargeAtlas,
    ),
    characterSmall: {
      body: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_body.png"),
        characterAtlas,
      ),
      eyes: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_eyes.png"),
        characterAtlas,
      ),
      hair: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_hair.png"),
        characterAtlas,
      ),
      underwear: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_underwear.png"),
        characterAtlas,
      ),
      dress: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_dress.png"),
        characterAtlas,
      ),
      armor: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_armor.png"),
        characterAtlas,
      ),
      binds: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_binds.png"),
        characterAtlas,
      ),
      expressions: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_small_expressions.png"),
        characterExpressionAtlas,
      ),
    },
    characterLarge: {
      body: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_body.png"),
        characterLargeAtlas,
      ),
      eyes: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_eyes.png"),
        characterLargeAtlas,
      ),
      hair: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_hair.png"),
        characterLargeAtlas,
      ),
      underwear: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_underwear.png"),
        characterLargeAtlas,
      ),
      dress: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_dress.png"),
        characterLargeAtlas,
      ),
      armor: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_armor.png"),
        characterLargeAtlas,
      ),
      binds: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_binds.png"),
        characterLargeAtlas,
      ),
      expressions: new PIXI.Spritesheet(
        await PIXI.Assets.load("images/character_large_expressions.png"),
        characterLargeExpressionAtlas,
      ),
    },
  };
  await Promise.all([
    data.tilesSmall.parse(),
    data.tilesLarge.parse(),
    data.characterSmall.body.parse(),
    data.characterSmall.eyes.parse(),
    data.characterSmall.hair.parse(),
    data.characterSmall.underwear.parse(),
    data.characterSmall.dress.parse(),
    data.characterSmall.armor.parse(),
    data.characterSmall.binds.parse(),
    data.characterSmall.expressions.parse(),
    data.characterLarge.body.parse(),
    data.characterLarge.eyes.parse(),
    data.characterLarge.hair.parse(),
    data.characterLarge.underwear.parse(),
    data.characterLarge.dress.parse(),
    data.characterLarge.armor.parse(),
    data.characterLarge.binds.parse(),
    data.characterLarge.expressions.parse(),
  ]);
  return data;
}
