import { Game } from "../engine";
import { EntityType } from "../entities";
import { createCampEntity } from "../entities/camp";
import { createMapEntity } from "../entities/map";
import { createPlayerEntity } from "../entities/player";
import { createHouseEntity } from "../entities/town";
import { createTreeEntity } from "../entities/tree";
import { createRandomMap } from "../map";
import { spawnNewDelivery } from "../systems/controller-spawner-delivery-system";

export function createMainScene(game: Game) {
  const map = createRandomMap();
  let towns: EntityType[] = [];
  for (const data of map.towns) {
    const town = createHouseEntity(game, data, map.tileSize);
    towns.push(town);
  }
  const graph: Array<[string, string]> = map.graph.map(([t1, t2]) => [
    towns[map.towns.indexOf(t1)].id,
    towns[map.towns.indexOf(t2)].id,
  ]);
  game.gameData.entities.push(
    createMapEntity(map, graph, game.images.tilesLarge),
  );
  game.gameData.entities.push(...towns);

  for (const data of map.camps)
    game.gameData.entities.push(createCampEntity(game, data, map.tileSize));
  for (const data of map.trees)
    game.gameData.entities.push(createTreeEntity(game, data, map.tileSize));
  game.gameData.entities.push(
    createPlayerEntity(
      map.towns[0].x + 1,
      map.towns[0].y + 1,
      game.images,
      map.tileSize,
    ),
  );
  spawnNewDelivery(game, towns[0]);
}
