import { ControllerSystemMethod, Game } from "../engine";
import { EntityType } from "../entities";
import { distanceBetween } from "../utils/analyzer";
import { getTilePathFromStartToFinish } from "../utils/pathing";

export function controllerAIAllySystem(): ControllerSystemMethod {
  return (game) => {
    for (const entity of game.gameData.entities) {
      if (!entity.aiAlly) continue;

      checkRetargetToNearestTown(game, entity);
      checkStruggle(entity);
      checkTargetTownArrival(game, entity);
    }
  };
}

function checkStruggle(entity: EntityType) {
  if (!entity.aiAlly) return;
  if (!entity.capturable) return;
  if (!entity.mapMovement) return;
  const capturable = entity.capturable;
  const movement = entity.mapMovement;

  if (capturable.state === "grabbed" || capturable.state === "bound") {
    movement.enabled = false;
  }
}

function checkRetargetToNearestTown(game: Game, entity: EntityType) {
  if (!entity.aiAlly) return;
  if (!entity.inventory) return;
  if (!entity.mapMovement) return;
  if (!entity.mapPosition) return;
  if (!entity.mapPathing) return;

  const ai = entity.aiAlly;
  const inventory = entity.inventory;
  const movement = entity.mapMovement;
  const pathing = entity.mapPathing;
  const position = entity.mapPosition;

  if (!pathing.enabled) return;

  if (ai.hasRetargetedNearestTown) {
    movement.speed = ai.escapeSpeed;
    return;
  }

  if (inventory.clothes === "underwear") {
    const map = game.gameData.entities.find((e) => e.mapMetadata);
    if (!map) return;
    if (!map.mapMetadata) return;

    movement.speed = ai.escapeSpeed;

    let targets = [];
    for (const entity of game.gameData.entities) {
      if (!entity.capturePrevention) continue;
      if (!entity.mapPosition) continue;
      const distance = distanceBetween(position, entity.mapPosition);
      targets.push({ entity, distance });
    }

    targets.sort((t1, t2) => t1.distance - t2.distance);

    for (const { entity } of targets) {
      if (!entity.mapPosition) continue;
      const target = {
        x: Math.round(entity.mapPosition.x - 0.8),
        y: Math.round(entity.mapPosition.y + 0.5),
      };
      const path = getTilePathFromStartToFinish(
        map.mapMetadata.roads,
        { x: Math.round(position.x), y: Math.round(position.y) },
        target,
      );
      if (path.length < 1) continue;
      const lastPath = path[path.length - 1];
      if (lastPath.x !== target.x || lastPath.y !== target.y) continue;

      pathing.path = path;
      pathing.step = 0;
      pathing.direction = "normal";
      movement.x = path[0].x;
      movement.y = path[0].y;
      ai.targetTown = entity.id;
      ai.hasRetargetedNearestTown = true;
      return;
    }
    return;
  }
}

function checkTargetTownArrival(game: Game, entity: EntityType) {
  if (!entity.aiAlly) return;
  if (!entity.mapPosition) return;
  const ai = entity.aiAlly;
  const position = entity.mapPosition;

  const targetTownId = ai.targetTown;
  const targetTown = game.gameData.entities.find((e) => e.id === targetTownId);
  if (!targetTown) return;
  if (!targetTown.mapPosition) return;
  if (Math.round(targetTown.mapPosition.y) !== Math.round(position.y)) return;
  if (targetTown.mapPosition.x < position.x - 1) return;
  if (targetTown.mapPosition.x > position.x + 1) return;

  game.gameData.stats.gatheredTownieMoney += entity.inventory?.money || 0;
  entity.deletionFlag = true;
}
