import { ControllerSystemMethod } from "../engine";
import {
  distanceBetween,
  getDirectionFromTargetToTarget,
} from "../utils/analyzer";
import { getRandomArrayItem } from "../utils/random";

export function controllerAIDeliverySystem(): ControllerSystemMethod {
  return (game, elapsedTicks) => {
    for (const entity of game.gameData.entities) {
      if (!entity.aiDelivery) continue;
      if (!entity.mapPosition) continue;
      if (!entity.characterSpriteRenderUpdater) continue;
      const ai = entity.aiDelivery;
      const position = entity.mapPosition;
      const render = entity.characterSpriteRenderUpdater;
      const players = game.gameData.entities.filter(
        (e) =>
          e.controlsPlayer &&
          e.mapPosition &&
          distanceBetween(position, e.mapPosition) < ai.playerFocusDistance,
      );
      if (players.length === 0) {
        ai.remainingRotationTime -= elapsedTicks;
        if (ai.remainingRotationTime > 0) continue;
        ai.remainingRotationTime = ai.rotationTime;
        render.direction = getRandomArrayItem(["n", "s", "e", "w"]);
        continue;
      }
      const alertPlayer = players.find(
        (p) =>
          distanceBetween(p.mapPosition!, position) < ai.playerAlertDistance,
      );
      if (alertPlayer) {
        render.status = "warning";
      } else {
        render.status = undefined;
      }
      const focusPlayer = alertPlayer || players[0];
      if (!focusPlayer) continue;
      if (!focusPlayer.mapPosition) continue;
      const direction = getDirectionFromTargetToTarget(
        position,
        focusPlayer.mapPosition,
      );
      render.direction = direction;
    }
  };
}
