import { ControllerSystemMethod } from "../engine";

export function controllerCharacterMovementSystem(): ControllerSystemMethod {
  return (game, elapsedTicks) => {
    for (const entity of game.gameData.entities) {
      if (!entity.mapMovement) continue;
      if (!entity.mapPosition) continue;
      if (!entity.characterSpriteRenderUpdater) continue;

      const position = entity.mapPosition;
      const movement = entity.mapMovement;
      const render = entity.characterSpriteRenderUpdater;

      if (!movement.enabled) continue;

      if (position.x === movement.x && position.y === movement.y) {
        render.movement.isAnimated = false;
        render.movement.frame = 0;
      } else {
        render.movement.isAnimated = true;
        render.movement.animationSpeed = Math.max(4, 40 * movement.speed);
      }

      if (position.x < movement.x) {
        position.x = Math.min(
          position.x + movement.speed * elapsedTicks,
          movement.x,
        );
        render.direction = "e";
      } else if (position.x > movement.x) {
        position.x = Math.max(
          position.x - movement.speed * elapsedTicks,
          movement.x,
        );
        render.direction = "w";
      } else if (position.y < movement.y) {
        position.y = Math.min(
          position.y + movement.speed * elapsedTicks,
          movement.y,
        );
        render.direction = "s";
      } else if (position.y > movement.y) {
        position.y = Math.max(
          position.y - movement.speed * elapsedTicks,
          movement.y,
        );
        render.direction = "n";
      }

      movement.isOnPosition =
        position.x === movement.x && position.y === movement.y;
    }
  };
}
