import { ControllerSystemMethod } from "../engine";

export function controllerMapPathingSystem(): ControllerSystemMethod {
  return (game) => {
    for (const entity of game.gameData.entities) {
      if (!entity.mapMovement) continue;
      if (!entity.mapPathing) continue;

      const movement = entity.mapMovement;
      const pathing = entity.mapPathing;
      if (!movement.enabled) continue;
      if (!pathing.enabled) continue;

      // This stops the next path from being selected until we're on the right position
      if (!movement.isOnPosition) continue;

      const currentPoint = pathing.path[pathing.step];
      if (!currentPoint) return;
      const stepIncrement = pathing.direction === "normal" ? 1 : -1;
      const nextStep = pathing.step + stepIncrement;
      const nextPoint = pathing.path[nextStep];
      if (!nextPoint) return;

      pathing.step = nextStep;

      if (!pathing.path[pathing.step + stepIncrement]) {
        if (pathing.direction === "normal") {
          pathing.direction = "reverse";
        } else {
          pathing.direction = "normal";
        }
      }

      movement.x = nextPoint.x;
      movement.y = nextPoint.y;
    }
  };
}
