import { ControllerSystemMethod, Game } from "../engine";
import { EntityType } from "../entities";
import { createAllyEntity } from "../entities/ally";
import { getTilePathFromStartToFinish } from "../utils/pathing";
import {
  getProbabilityResult,
  getRandomArrayItem,
  getRandomIntegerInRange,
} from "../utils/random";

export function controllerSpawnerAllySystem(): ControllerSystemMethod {
  return (game, elapsedTicks) => {
    for (const entity of game.gameData.entities) {
      if (!entity.spawnerAlly) continue;
      entity.spawnerAlly.remainingTime -= elapsedTicks;
      if (entity.spawnerAlly.remainingTime < 0) {
        if (getProbabilityResult(entity.spawnerAlly.probability)) {
          spawnNewAlly(game, entity);
        }
        entity.spawnerAlly.remainingTime = getRandomIntegerInRange(
          entity.spawnerAlly.minTime,
          entity.spawnerAlly.maxTime,
        );
      }
    }
  };
}

function spawnNewAlly(game: Game, homeTown: EntityType) {
  if (!homeTown.mapPosition) return;
  const map = game.gameData.entities.find((e) => e.mapMetadata);
  if (!map) return;
  if (!map.mapMetadata) return;
  const targetTownId = getRandomArrayItem(
    map.mapMetadata.graph
      .filter(([t0, _]) => t0 == homeTown.id)
      .map(([_, t1]) => t1)
      .concat(
        map.mapMetadata.graph
          .filter(([_, t1]) => t1 == homeTown.id)
          .map(([t0, _]) => t0),
      ),
  );
  const targetTown = game.gameData.entities.find((e) => e.id === targetTownId);
  if (!targetTown) return;
  if (!targetTown.mapPosition) return;
  const path = getTilePathFromStartToFinish(
    map.mapMetadata.roads,
    {
      x: Math.round(homeTown.mapPosition.x - 0.8),
      y: Math.round(homeTown.mapPosition.y + 0.5),
    },
    {
      x: Math.round(targetTown.mapPosition.x - 0.8),
      y: Math.round(targetTown.mapPosition.y + 0.5),
    },
  );
  if (path.length < 2) return;

  game.gameData.entities.push(
    createAllyEntity(targetTown.id, path, map.mapMetadata.tileSize),
  );
}
