import { ControllerSystemMethod, Game } from "../engine";
import { EntityType } from "../entities";
import { createDeliveryGiverEntity } from "../entities/delivery-giver";
import {
  getProbabilityResult,
  getRandomArrayItem,
  getRandomIntegerInRange,
} from "../utils/random";

export function controllerSpawnerDeliverySystem(): ControllerSystemMethod {
  return (game, elapsedTicks) => {
    let hasAIDelivery = false;
    for (const entity of game.gameData.entities) {
      if (entity.aiDelivery) hasAIDelivery = true;
      if (!entity.spawnerDelivery) continue;
      entity.spawnerDelivery.remainingTime -= elapsedTicks;
      if (entity.spawnerDelivery.remainingTime < 0) {
        if (getProbabilityResult(entity.spawnerDelivery.probability)) {
          spawnNewDelivery(game, entity);
        }
        entity.spawnerDelivery.remainingTime = getRandomIntegerInRange(
          entity.spawnerDelivery.minTime,
          entity.spawnerDelivery.maxTime,
        );
      }
    }
    if (!hasAIDelivery) {
      const spawner = getRandomArrayItem(
        game.gameData.entities.filter((e) => e.spawnerDelivery),
      );
      if (!spawner?.spawnerDelivery) return;
      spawnNewDelivery(game, spawner);
    }
  };
}

export function spawnNewDelivery(game: Game, homeTown: EntityType) {
  if (!homeTown.mapPosition) return;
  const targetTown = getRandomArrayItem(
    game.gameData.entities.filter(
      (e) => e.spawnerDelivery && e.id !== homeTown.id,
    ),
  );
  if (!targetTown) return;
  const map = game.gameData.entities.find((e) => e.mapMetadata);
  if (!map) return;
  if (!map.mapMetadata) return;

  let deliveryGiver = game.gameData.entities.find(
    (e) =>
      e.interactibleNPC &&
      e.interactibleNPC.type === "pickUp" &&
      e.interactibleNPC.town === homeTown.id,
  );
  if (!deliveryGiver) {
    deliveryGiver = createDeliveryGiverEntity(
      homeTown.mapPosition.x - 0.8,
      homeTown.mapPosition.y + 0.5,
      targetTown.id,
      map.mapMetadata.tileSize,
    );
    game.gameData.entities.push(deliveryGiver);
  }

  if (deliveryGiver.inventory) {
    deliveryGiver.inventory.packages.push({
      name: "Small package",
      value: 10,
      targetId: targetTown.id,
    });
  }
}
