import { ControllerSystemMethod } from "../engine";

export function controllerSpriteCaptureSystem(): ControllerSystemMethod {
  return (game) => {
    for (const entity of game.gameData.entities) {
      if (!entity.capturable) continue;
      if (!entity.characterSpriteRenderUpdater) continue;

      const capturable = entity.capturable;
      const render = entity.characterSpriteRenderUpdater;

      if (capturable.state === "bound") {
        render.state = "struggle";
        render.movement.isAnimated = true;
        render.movement.animationSpeed = capturable.isStruggling ? 6 : 4;
        if (capturable.isStruggling) {
          render.status = "shouting";
        } else {
          render.status = undefined;
        }
      } else if (capturable.state === "grabbed") {
        render.state = undefined;
        render.movement.isAnimated = true;
        render.movement.animationSpeed = capturable.isStruggling ? 6 : 4;
        if (capturable.isStruggling) {
          render.status = "shouting";
        } else {
          render.status = undefined;
        }
      } else {
        render.state = undefined;
      }
    }
  };
}
