import { CharacterSpriteAnimationFrameMetadata } from "../components/character-sprite-render-updater";
import { ControllerSystemMethod } from "../engine";

export function controllerSpriteCharacterAnimationSystem(): ControllerSystemMethod {
  return (game, elapsedTicks) => {
    for (const entity of game.gameData.entities) {
      if (!entity.characterSpriteRenderUpdater) continue;
      updateSpriteTimeIfNeeded(
        elapsedTicks,
        entity.characterSpriteRenderUpdater.movement,
      );
      updateSpriteTimeIfNeeded(
        elapsedTicks,
        entity.characterSpriteRenderUpdater.expression,
      );
    }
  };
}

function updateSpriteTimeIfNeeded(
  timeSinceLastFrame: number,
  data: CharacterSpriteAnimationFrameMetadata,
) {
  if (!data.isAnimated) return;
  data.nextFrameIn -= timeSinceLastFrame * data.animationSpeed;
  while (data.nextFrameIn < 0) {
    data.frame++;
    if (data.frame > 3) data.frame = 0;
    data.nextFrameIn += 20;
  }
}
