import { ControllerSystemMethod } from "../engine";
import { EntityType } from "../entities";
import { distanceBetween } from "../utils/analyzer";

export function controllerUIPlayerSystem(): ControllerSystemMethod {
  return (game) => {
    for (const entity of game.gameData.entities) {
      if (!entity.uiPlayerUpdater) continue;
      if (!entity.mapPosition) continue;
      if (!entity.controlsPlayer) continue;

      const ui = entity.uiPlayerUpdater;
      const position = entity.mapPosition;

      ui.totalAcquiredMoney = game.gameData.stats.gatheredPlayerMoney;
      ui.totalTownieMoney = game.gameData.stats.gatheredTownieMoney;
      ui.totalStolenMoney = game.gameData.stats.gatheredEnemyMoney;
      ui.currentTowniesToSave = game.gameData.entities.filter(
        (e) =>
          e.capturable &&
          e.capturable.isFreeableByTownie &&
          e.capturable.state !== "free",
      ).length;
      ui.currentBanditsToSave = game.gameData.entities.filter(
        (e) =>
          e.capturable &&
          e.capturable.isFreeableByBandits &&
          e.capturable.state !== "free",
      ).length;

      function getDirectionTowards(target: EntityType | undefined): number {
        if (!target) return 0;
        if (!target.mapPosition) return 0;
        const targetPosition = target.mapPosition;
        return Math.atan2(
          position.x - targetPosition.x,
          targetPosition.y - position.y,
        );
      }

      if (entity.inventory) {
        if (entity.controlsPlayer && entity.inventory.clothes !== "underwear") {
          ui.canCapture = true;
          ui.captureAttemptPercent =
            1 -
            entity.controlsPlayer.captureAttempt.current /
              entity.controlsPlayer.captureAttempt.time;
        } else {
          ui.canCapture = false;
        }
        if (entity.inventory.packages.length >= 1) {
          ui.delivery = {
            state: "delivery",
            packages: entity.inventory.packages.map((p) => ({
              name: p.name,
              value: p.value,
              direction: getDirectionTowards(
                game.gameData.entities.find((e) => e.id == p.targetId),
              ),
            })),
          };
        } else {
          let closestDeliveryPerson:
            | { entity: EntityType; distance: number }
            | undefined = undefined;
          for (const entity of game.gameData.entities) {
            if (entity.type !== "deliveryGiver") continue;
            if (!entity.mapPosition) continue;
            const targetPosition = entity.mapPosition;
            const distance = distanceBetween(position, targetPosition);
            if (!closestDeliveryPerson)
              closestDeliveryPerson = { entity, distance };
            if (closestDeliveryPerson.distance > distance)
              closestDeliveryPerson = { entity, distance };
          }
          if (closestDeliveryPerson) {
            ui.delivery = {
              state: "noDelivery",
              direction: getDirectionTowards(closestDeliveryPerson?.entity),
            };
          } else {
            ui.delivery = { state: "noDeliveryNoGiver" };
          }
        }
      } else {
        ui.delivery = {
          state: "noDelivery",
          direction: 0,
        };
      }
    }
  };
}
