import { SystemMethod } from "../engine";

export function rendererUIPlayerSystem(): SystemMethod {
  return (game) => {
    for (const entity of game.gameData.entities) {
      if (!entity.uiPlayerUpdater) continue;
      const ui = entity.uiPlayerUpdater;

      if (ui.showBanditUI) {
        ui.pixi.acquiredMoney.text = `Stolen money (player): ${ui.totalAcquiredMoney} gold`;
        ui.pixi.allyMoney.text = `Gathered money (townies): ${ui.totalTownieMoney} gold`;
        ui.pixi.stolenMoney.text = `Stolen money (allies): ${ui.totalStolenMoney} gold`;
        ui.pixi.alliesToSave.text = `Bandits to rescue: ${ui.currentBanditsToSave}`;
      } else {
        ui.pixi.acquiredMoney.text = `Gathered money (player): ${ui.totalAcquiredMoney} gold`;
        ui.pixi.allyMoney.text = `Gathered money (allies): ${ui.totalTownieMoney} gold`;
        ui.pixi.stolenMoney.text = `Stolen money: ${ui.totalStolenMoney} gold`;
        ui.pixi.alliesToSave.text = `Allies to rescue: ${ui.currentTowniesToSave}`;
      }

      if (ui.canCapture) {
        ui.pixi.noCaptureContainer.visible = false;
        ui.pixi.captureContainer.visible = true;
        ui.pixi.captureContainerPercentBar.clear();
        ui.pixi.captureContainerPercentBar.rect(
          4,
          4,
          192 * ui.captureAttemptPercent,
          12,
        );
        ui.pixi.captureContainerPercentBar.fill(0x0000cc);
      } else {
        ui.pixi.noCaptureContainer.visible = true;
        ui.pixi.captureContainer.visible = false;
      }

      if (ui.delivery.state === "noDeliveryNoGiver") {
        ui.pixi.deliveryContainer.visible = false;
        ui.pixi.noDeliveryContainer.visible = false;
      } else if (ui.delivery.state === "noDelivery") {
        ui.pixi.deliveryContainer.visible = false;
        ui.pixi.noDeliveryContainer.visible = true;
        ui.pixi.noDeliveryArrow.rotation = ui.delivery.direction;
      } else if (ui.delivery.state === "delivery") {
        ui.pixi.deliveryContainer.visible = true;
        ui.pixi.noDeliveryContainer.visible = false;
        let packageText = `Packages:\n`;
        for (const delivery of ui.delivery.packages)
          packageText += ` - ${delivery.name} (worth ${delivery.value} gold)\n`;
        ui.pixi.deliveryPackageText.text = packageText;
        for (const [index, arrow] of ui.pixi.deliveryArrows.entries()) {
          let packageData = ui.delivery.packages[index];
          if (!packageData) {
            arrow.visible = false;
            continue;
          }
          arrow.visible = true;
          arrow.rotation = packageData.direction;
        }
      }
    }
  };
}
