import { assertNever } from "./lang";

export function distanceBetween(
  dot1: { readonly x: number; readonly y: number },
  dot2: { readonly x: number; readonly y: number },
): number {
  return Math.sqrt(
    (dot1.x - dot2.x) * (dot1.x - dot2.x) +
      (dot1.y - dot2.y) * (dot1.y - dot2.y),
  );
}

export function getDirectionFromTargetToTarget(
  origin: { x: number; y: number },
  target: { x: number; y: number },
): "n" | "s" | "e" | "w" {
  const xDiff = Math.abs(origin.x - target.x);
  const yDiff = Math.abs(origin.y - target.y);
  if (xDiff > yDiff) {
    if (origin.x < target.x) return "e";
    if (origin.x > target.x) return "w";
  } else {
    if (origin.y < target.y) return "s";
    if (origin.y > target.y) return "n";
  }
  return "s";
}

export function getIsReachableMethod(
  position: { x: number; y: number },
  direction: "n" | "s" | "e" | "w",
  frontDistance: number,
  sideDistance: number,
  backDistance: number,
) {
  return (otherPosition: { x: number; y: number }) => {
    if (direction === "n") {
      return (
        otherPosition.x > position.x - sideDistance &&
        otherPosition.x < position.x + sideDistance &&
        otherPosition.y > position.y - frontDistance &&
        otherPosition.y < position.y + backDistance
      );
    }
    if (direction === "s") {
      return (
        otherPosition.x > position.x - sideDistance &&
        otherPosition.x < position.x + sideDistance &&
        otherPosition.y > position.y - backDistance &&
        otherPosition.y < position.y + frontDistance
      );
    }
    if (direction === "e") {
      return (
        otherPosition.x > position.x - backDistance &&
        otherPosition.x < position.x + frontDistance &&
        otherPosition.y > position.y - sideDistance &&
        otherPosition.y < position.y + sideDistance
      );
    }
    if (direction === "w") {
      return (
        otherPosition.x > position.x - frontDistance &&
        otherPosition.x < position.x + backDistance &&
        otherPosition.y > position.y - sideDistance &&
        otherPosition.y < position.y + sideDistance
      );
    }
    assertNever(direction, "direction");
  };
}
