export type PathData = {
  x: number;
  y: number;
};

export function getTilePathFromStartToFinish(
  roads: Array<{ x: number; y: number }>,
  start: { readonly x: number; readonly y: number },
  end: { readonly x: number; readonly y: number },
): PathData[] {
  let currentRoad = roads.find((r) => r.x === start.x && r.y === start.y);
  if (!currentRoad) return [];
  const path: PathData[] = [];
  path.push({ x: currentRoad.x, y: currentRoad.y });
  while (currentRoad) {
    const nextPositions: Array<[number, number, number]> = [];
    if (end.x < currentRoad.x)
      nextPositions.push([
        currentRoad.x - 1,
        currentRoad.y,
        currentRoad.x - end.x,
      ]);
    if (end.x > currentRoad.x)
      nextPositions.push([
        currentRoad.x + 1,
        currentRoad.y,
        end.x - currentRoad.x,
      ]);
    if (end.y < currentRoad.y)
      nextPositions.push([
        currentRoad.x,
        currentRoad.y - 1,
        currentRoad.y - end.y,
      ]);
    if (end.y > currentRoad.y)
      nextPositions.push([
        currentRoad.x,
        currentRoad.y + 1,
        end.y - currentRoad.y,
      ]);
    nextPositions.sort((a, b) => b[2] - a[2]);
    const nextRoad = (() => {
      for (const [x, y] of nextPositions) {
        for (const r of roads) {
          if (r.x === x && r.y === y) return r;
        }
      }
      return;
    })();
    if (nextRoad) path.push({ x: nextRoad.x, y: nextRoad.y });
    currentRoad = nextRoad;
  }

  return path;
}
